==> apache-iotdb-2.0.2-1-source-release.zip: apache-iotdb-2.0.2-1/iotdb-core/datanode/src/main/java/org/apache/iotdb/db/queryengine/plan/relational/sql/ast/SortItem.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.iotdb.db.queryengine.plan.relational.sql.ast;

import com.google.common.collect.ImmutableList;

import java.util.List;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;
import static java.util.Objects.requireNonNull;

public class SortItem extends Node {

  public enum Ordering {
    ASCENDING,
    DESCENDING
  }

  public enum NullOrdering {
    FIRST,
    LAST,
    UNDEFINED
  }

  private final Expression sortKey;
  private final Ordering ordering;
  private final NullOrdering nullOrdering;

  public SortItem(Expression sortKey, Ordering ordering, NullOrdering nullOrdering) {
    super(null);
    this.ordering = ordering;
    this.sortKey = sortKey;
    this.nullOrdering = nullOrdering;
  }

  public SortItem(
      NodeLocation location, Expression sortKey, Ordering ordering, NullOrdering nullOrdering) {
    super(requireNonNull(location, "location is null"));
    this.ordering = ordering;
    this.sortKey = sortKey;
    this.nullOrdering = nullOrdering;
  }

  public Expression getSortKey() {
    return sortKey;
  }

  public Ordering getOrdering() {
    return ordering;
  }

  public NullOrdering getNullOrdering() {
    return nullOrdering;
  }

  @Override
  public <R, C> R accept(AstVisitor<R, C> visitor, C context) {
    return visitor.visitSortItem(this, context);
  }

  @Override
  public List<Node> getChildren() {
    return ImmutableList.of(sortKey);
  }

  @Override
  public String toString() {
    return toStringHelper(this)
        .add("sortKey", sortKey)
        .add("ordering", ordering)
        .add("nullOrdering", nullOrdering)
        .toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }

    SortItem sortItem = (SortItem) o;
    return Objects.equals(sortKey, sortItem.sortKey)
        && (ordering == sortItem.ordering)
        && (nullOrdering == sortItem.nullOrdering);
  }

  @Override
  public int hashCode() {
    return Objects.hash(sortKey, ordering, nullOrdering);
  }

  @Override
  public boolean shallowEquals(Node other) {
    if (!sameClass(this, other)) {
      return false;
    }

    SortItem otherItem = (SortItem) other;
    return ordering == otherItem.ordering && nullOrdering == otherItem.nullOrdering;
  }
}

==> apache-iotdb-2.0.5-source-release.zip: apache-iotdb-2.0.5/iotdb-core/datanode/src/main/java/org/apache/iotdb/db/queryengine/plan/relational/sql/ast/SortItem.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.iotdb.db.queryengine.plan.relational.sql.ast;

import com.google.common.collect.ImmutableList;
import org.apache.tsfile.utils.ReadWriteIOUtils;

import java.io.DataOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.List;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;
import static java.util.Objects.requireNonNull;

public class SortItem extends Node {

  public enum Ordering {
    ASCENDING,
    DESCENDING
  }

  public enum NullOrdering {
    FIRST,
    LAST,
    UNDEFINED
  }

  private final Expression sortKey;
  private final Ordering ordering;
  private final NullOrdering nullOrdering;

  public SortItem(Expression sortKey, Ordering ordering, NullOrdering nullOrdering) {
    super(null);
    this.ordering = ordering;
    this.sortKey = sortKey;
    this.nullOrdering = nullOrdering;
  }

  public SortItem(
      NodeLocation location, Expression sortKey, Ordering ordering, NullOrdering nullOrdering) {
    super(requireNonNull(location, "location is null"));
    this.ordering = ordering;
    this.sortKey = sortKey;
    this.nullOrdering = nullOrdering;
  }

  public Expression getSortKey() {
    return sortKey;
  }

  public Ordering getOrdering() {
    return ordering;
  }

  public NullOrdering getNullOrdering() {
    return nullOrdering;
  }

  @Override
  public <R, C> R accept(AstVisitor<R, C> visitor, C context) {
    return visitor.visitSortItem(this, context);
  }

  @Override
  public List<Node> getChildren() {
    return ImmutableList.of(sortKey);
  }

  @Override
  public String toString() {
    return toStringHelper(this)
        .add("sortKey", sortKey)
        .add("ordering", ordering)
        .add("nullOrdering", nullOrdering)
        .toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }

    SortItem sortItem = (SortItem) o;
    return Objects.equals(sortKey, sortItem.sortKey)
        && (ordering == sortItem.ordering)
        && (nullOrdering == sortItem.nullOrdering);
  }

  @Override
  public int hashCode() {
    return Objects.hash(sortKey, ordering, nullOrdering);
  }

  @Override
  public boolean shallowEquals(Node other) {
    if (!sameClass(this, other)) {
      return false;
    }

    SortItem otherItem = (SortItem) other;
    return ordering == otherItem.ordering && nullOrdering == otherItem.nullOrdering;
  }

  void serialize(ByteBuffer buffer) {
    Expression.serialize(sortKey, buffer);
    ReadWriteIOUtils.write((byte) ordering.ordinal(), buffer);
    ReadWriteIOUtils.write((byte) nullOrdering.ordinal(), buffer);
  }

  void serialize(DataOutputStream stream) throws IOException {
    Expression.serialize(sortKey, stream);
    ReadWriteIOUtils.write((byte) ordering.ordinal(), stream);
    ReadWriteIOUtils.write((byte) nullOrdering.ordinal(), stream);
  }

  public SortItem(ByteBuffer byteBuffer) {
    super(null);
    this.sortKey = Expression.deserialize(byteBuffer);

    ordering = Ordering.values()[ReadWriteIOUtils.readByte(byteBuffer)];
    nullOrdering = NullOrdering.values()[ReadWriteIOUtils.readByte(byteBuffer)];
  }
}
